<?php
/*
Plugin Name: Pre Date Future Post
Plugin URI: http://wordpress.org/extend/plugins/pre-date-future-post/
Description: Allows you to set posts to appear in the future, but have a post date in the past.
Author: Brodie Thiesfield
Version: 1.0
Author URI: http://jellycan.com/
*/

/** 
 * Debug logging
 */
function append_log($line) {
    $fp = fopen(__FILE__ . ".log", "ab");
    if ($fp) {
        fwrite($fp, date("Y-m-d h:i:s ").$line."\n");
        fclose($fp);
    }
}

/** 
 * Function that does the actual posting - called by wp_cron
 */
function predatefuturepost_update_post() {
    //append_log("predatefuturepost_update_post");
    
	global $wpdb;
    $sql =  "SELECT post_id " .
            "FROM wp_posts JOIN wp_postmeta ON ID = post_id " .
            "WHERE post_type = 'post' " .
            "AND post_status = 'pending' " .
            "AND meta_key = 'make-public-date' " .
            "AND meta_value < " . mktime();
    //append_log("predatefuturepost_update_post: $sql");
	$result = $wpdb->get_results($sql);

    foreach ($result as $p) {
        append_log("predatefuturepost_update_post: publishing post_id $p->post_id");
        wp_update_post(array('ID' => $p->post_id, 'post_status' => 'publish'));
        delete_post_meta($p->post_id, 'make-public-date');
	}
}
add_action ('predatefuturepost_update_post_'.$current_blog->blog_id, 'predatefuturepost_update_post');

/** 
 * Called at plugin activation
 */
function predatefuturepost_activate () {
    append_log("predatefuturepost_activate");

	global $current_blog;
	wp_schedule_event(time() + 5, 'hourly', 'predatefuturepost_update_post_'.$current_blog->blog_id);
}
register_activation_hook (__FILE__, 'predatefuturepost_activate');

/**
 * Called at plugin deactivation
 */
function predatefuturepost_deactivate () {
    append_log("predatefuturepost_deactivate");

	global $current_blog;
	wp_clear_scheduled_hook('predatefuturepost_update_post_'.$current_blog->blog_id);
}
register_deactivation_hook (__FILE__, 'predatefuturepost_deactivate');

/**
 * adds an 'Make Public' column to the post display table.
 */
function predatefuturepost_add_column ($columns) {
  	$columns['predatefuturepost'] = 'Make Public';
  	return $columns;
}
add_filter ('manage_posts_columns', 'predatefuturepost_add_column');
add_filter ('manage_pages_columns', 'predatefuturepost_add_column');

/**
 * fills the 'Make Public' column of the post display table.
 */
function predatefuturepost_show_value ($column_name) {
	if ($column_name === 'predatefuturepost') {
    	global $wpdb, $post;
        $tbd = $wpdb->get_var("SELECT meta_value FROM $wpdb->postmeta WHERE meta_key = 'make-public-date' AND post_id = $post->ID");
        echo ($tbd ? date('d M y', $tbd) : '');
  	}
}
add_action ('manage_posts_custom_column', 'predatefuturepost_show_value');
add_action ('manage_pages_custom_column', 'predatefuturepost_show_value');

/**
 * Add's hooks to get the meta box added to post
 */
function predatefuturepost_meta_post() {
    global $wpdb, $post;
    $status = $wpdb->get_var("SELECT post_status FROM $wpdb->posts WHERE ID = $post->ID");
    if ($status === 'pending') {
        add_meta_box('predatefuturepostdiv', __('Make Post Public Date'), 'predatefuturepost_meta_box', 'post', 'advanced', 'high');
    }
}
add_action ('dbx_post_advanced','predatefuturepost_meta_post');

/**
 * Add's hooks to get the meta box added to page
 */
function predatefuturepost_meta_page() {
    global $wpdb, $post;
    $status = $wpdb->get_var("SELECT post_status FROM $wpdb->posts WHERE ID = $post->ID");
    if ($status === 'pending') {
    	add_meta_box('predatefuturepostdiv', __('Make Post Public Date'), 'predatefuturepost_meta_box', 'page', 'advanced', 'high');
    }
}
add_action ('edit_page_form','predatefuturepost_meta_page');

/**
 * Actually adds the meta box
 */
function predatefuturepost_meta_box($post) { 
	// Get default month
	$ts = get_post_meta($post->ID, 'make-public-date', true);
	if (empty($ts)) {
		$defaultday = date('d');
		$defaultmonth = date('F');
		$defaultyear = date('Y');
        $checked = "";
	} else {
		$defaultday = date('d', $ts);
		$defaultmonth = date('F', $ts);
		$defaultyear = date('Y', $ts);
        $checked = "checked='checked'";
	}

	$rv = array();
	$rv[] = "<p><input type='checkbox' name='predatefuturepost_enable' id='predatefuturepost_enable' value='on' $checked />";
	$rv[] = '<label for="predatefuturepost_enable">Automatically publish this post on the following date</label></p>';
	$rv[] = '<table><tr>';
	   $rv[] = '<th style="text-align: left;">Day</th>';
	   $rv[] = '<th style="text-align: left;">Month</th>';
	   $rv[] = '<th style="text-align: left;">Year</th>';
	$rv[] = '</tr><tr>';
	$rv[] = '<td>';
	$rv[] = "<input type='text' id='predatefuturepost_day' name='predatefuturepost_day' value='$defaultday' size='2' />"; 
	$rv[] = '</td><td>'; 
	$rv[] = '<select name="predatefuturepost_month" id="predatefuturepost_month">';
	for($i = 1; $i <= 12; $i++) {
		if ($defaultmonth == date('F',mktime(0, 0, 0, $i, 1, date("Y"))))
			$selected = ' selected="selected"';
		else
			$selected = '';
		$rv[] = '<option value="'.date('m',mktime(0, 0, 0, $i, 1, date("Y"))).'"'.$selected.'>'.date('F',mktime(0, 0, 0, $i, 1, date("Y"))).'</option>';
	}
	$rv[] = '</select>';
	$rv[] = '</td><td>';
	$rv[] = '<select name="predatefuturepost_year" id="predatefuturepost_year">';
	$currentyear = date('Y');
	if ($defaultyear < $currentyear)
		$currentyear = $defaultyear;
	for($i = $currentyear; $i < $currentyear + 8; $i++) {
		if ($i == $defaultyear)
			$selected = ' selected="selected"';
		else
			$selected = '';
		$rv[] = '<option'.$selected.'>'.($i).'</option>';
	}
	$rv[] = '</select>';
	$rv[] = '</td></tr></table>';

	echo implode("\n",$rv);
}

/**
 * Called when post is saved - stores make-public-date meta value
 */
function predatefuturepost_update_post_meta($id) {
    if (array_key_exists('predatefuturepost_enable', $_POST) && $_POST['predatefuturepost_enable'] == 'on') {
        $day   = $_POST['predatefuturepost_day'];
        $month = $_POST['predatefuturepost_month'];
        $year  = $_POST['predatefuturepost_year'];

        $ts = mktime(0,0,0,$month,$day,$year);
        delete_post_meta($id, 'make-public-date');
        update_post_meta($id, 'make-public-date', $ts, true);
    }
    else {
        delete_post_meta($id, 'make-public-date');
    }
}
add_action('save_post','predatefuturepost_update_post_meta');

?>
